@@ -22,6 +22,7 @@
 #include <config.h>
 #include <libgnome/libgnome.h>
 #include <sys/types.h>
+#include <sys/socket.h>
 #include <unistd.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -165,6 +166,7 @@ gboolean
 gdm_auth_secure_display (GdmDisplay *d)
 {
     FILE *af, *af_gdm;
+    int closeret;
 
     if G_UNLIKELY (!d)
 	return FALSE;
@@ -201,7 +203,7 @@ gdm_auth_secure_display (GdmDisplay *d)
 	    /* Make it owned by the user that Xnest is started as */
 	    fchown (authfd, d->server_uid, -1);
 
-	    af = fdopen (authfd, "w");
+	    VE_IGNORE_EINTR (af = fdopen (authfd, "w"));
 
 	    if G_UNLIKELY (af == NULL) {
 		    g_free (d->authfile);
@@ -223,7 +225,7 @@ gdm_auth_secure_display (GdmDisplay *d)
 		    d->authfile_gdm = NULL;
 		    g_free (d->authfile);
 		    d->authfile = NULL;
-		    fclose (af);
+		    VE_IGNORE_EINTR (fclose (af));
 		    return FALSE;
 	    }
     } else {
@@ -276,12 +278,14 @@ gdm_auth_secure_display (GdmDisplay *d)
 
     gdm_debug ("gdm_auth_secure_display: Setting up access");
 
-    if G_UNLIKELY (fclose (af) < 0) {
+    VE_IGNORE_EINTR (closeret = fclose (af));
+    if G_UNLIKELY (closeret < 0) {
 	    display_add_error (d);
 	    return FALSE;
     }
     if (af_gdm != NULL) {
-	    if G_UNLIKELY (fclose (af_gdm) < 0) {
+	    VE_IGNORE_EINTR (closeret = fclose (af_gdm));
+	    if G_UNLIKELY (closeret < 0) {
 		    display_add_error (d);
 		    return FALSE;
 	    }
@@ -300,6 +304,9 @@ get_local_auths (GdmDisplay *d)
 {
     gboolean is_local = FALSE;
     const char lo[] = {127,0,0,1};
+#ifdef ENABLE_IPV6
+    const char lo6[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1};
+#endif
     guint i;
     const GList *local_addys = NULL;
     gboolean added_lo = FALSE;
@@ -323,12 +330,33 @@ get_local_auths (GdmDisplay *d)
 	    is_local = TRUE;
     } else  {
 	    is_local = FALSE;
-	    if (gdm_is_local_addr (&(d->addr)))
-		    is_local = TRUE;
-	    for (i = 0; ! is_local && i < d->addr_count; i++) {
-		    if (gdm_is_local_addr (&(d->addrs[i]))) {
+#ifdef ENABLE_IPV6
+	    if (d->addrtype == AF_INET6) {
+		    if (gdm_is_local_addr6 (&(d->addr6)))
 			    is_local = TRUE;
-			    break;
+	    }
+	    else
+#endif
+	    {
+		    if (gdm_is_local_addr (&(d->addr)))
+			    is_local = TRUE;
+	    }
+
+	    for (i = 0; ! is_local && i < d->addr_count; i++) {
+#ifdef ENABLE_IPV6
+		    if (d->addrs[i].ss_family == AF_INET6) {
+			    if (gdm_is_local_addr6 (&((struct sockaddr_in6 *)(&(d->addrs[i])))->sin6_addr)) {
+				    is_local = TRUE;
+				    break;
+			    }
+		    }
+		    else
+#endif
+		    {
+			    if (gdm_is_local_addr (&((struct sockaddr_in *)(&(d->addrs[i])))->sin_addr)) {
+				    is_local = TRUE;
+				    break;
+			    }
 		    }
 	    }
     }
@@ -374,51 +402,100 @@ get_local_auths (GdmDisplay *d)
     if ( ! SERVER_IS_LOCAL (d)) {
 	    /* we should write out an entry for d->addr since
 	       possibly it is not in d->addrs */
-	    if ( ! add_auth_entry (d, &auths, NULL, NULL, FamilyInternet,
-				   (char *)&(d->addr), 4))
+#ifdef ENABLE_IPV6
+	    if ( ! add_auth_entry (d, &auths, NULL, NULL, FamilyInternet, (char *)((d->addr6).s6_addr), sizeof (struct in6_addr)))
+		    goto get_local_auth_error;
+	    else
+#endif
+	    if ( ! add_auth_entry (d, &auths, NULL, NULL, FamilyInternet, (char *)&(d->addr), sizeof (struct in_addr)))
 		    goto get_local_auth_error;
 
-	    if (gdm_is_loopback_addr (&(d->addr)))
-		    added_lo = TRUE;
+#ifdef ENABLE_IPV6
+	    if (d->addrtype == AF_INET6 &&  gdm_is_loopback_addr6 (&(d->addr6)))
+		added_lo = TRUE;
+#endif
+	    if (d->addrtype == AF_INET && gdm_is_loopback_addr (&(d->addr)))
+		added_lo = TRUE;
     }
     
     /* Network access: Write out an authentication entry for each of
      * this host's official addresses */
     for (i = 0; i < d->addr_count; i++) {
-	    struct in_addr *ia = &(d->addrs[i]);
+#ifdef ENABLE_IPV6
+	    struct in6_addr *ia6;
+#endif   
+	    struct in_addr *ia;
 
-	    if (memcmp (ia, &(d->addr), sizeof (struct in_addr)) == 0)
-		    continue;
+#ifdef ENABLE_IPV6
+	    if (d->addrs[i].ss_family == AF_INET6) {
+		    ia6 = &(((struct sockaddr_in6 *)(&(d->addrs[i])))->sin6_addr);
+		    if (memcmp (ia6, &(d->addr6), sizeof (struct in6_addr)) == 0)
+			    continue;
 
-	    if ( ! add_auth_entry (d, &auths, NULL, NULL, FamilyInternet,
-				   (char *)&ia->s_addr, 4))
-		    goto get_local_auth_error;
+		    if ( ! add_auth_entry (d, &auths, NULL, NULL, FamilyInternet, (char *)(ia6->s6_addr), sizeof (struct in6_addr)))
+			    goto get_local_auth_error;
 
-	    if (gdm_is_loopback_addr (ia))
-		    added_lo = TRUE;
+		    if (gdm_is_loopback_addr6 (ia6))
+			    added_lo = TRUE;
+	    }
+	    else
+#endif   
+	    {
+		    ia = &(((struct sockaddr_in *)(&(d->addrs[i])))->sin_addr);
+		    if (memcmp (ia, &(d->addr), sizeof (struct in_addr)) == 0)
+			    continue;
+
+		    if ( ! add_auth_entry (d, &auths, NULL, NULL, FamilyInternet, (char *)&ia->s_addr, sizeof (struct in_addr)))
+			    goto get_local_auth_error;
+
+		    if (gdm_is_loopback_addr (ia))
+			    added_lo = TRUE;
+	    }
     }
 
     /* Network access: Write out an authentication entry for each of
      * this host's local addresses if any */
     for (; local_addys != NULL; local_addys = local_addys->next) {
-	    struct in_addr *ia = (struct in_addr *) local_addys->data;
+	    struct sockaddr *ia = (struct sockaddr *) local_addys->data;
 
 	    if (ia == NULL)
 		    break;
 
-	    if ( ! add_auth_entry (d, &auths, NULL, NULL, FamilyInternet,
-				   (char *)&ia->s_addr, 4))
-		    goto get_local_auth_error;
+#ifdef ENABLE_IPV6
+	    if (ia->sa_family == AF_INET6) {
+		    if ( ! add_auth_entry (d, &auths, NULL, NULL, FamilyInternet, (char *)(((struct sockaddr_in6 *)ia)->sin6_addr.s6_addr), sizeof (struct in6_addr)))
+			    goto get_local_auth_error;
 
-	    if (gdm_is_loopback_addr (ia))
-		    added_lo = TRUE;
+		    if (gdm_is_loopback_addr6 (&((struct sockaddr_in6 *)ia)->sin6_addr))
+			    added_lo = TRUE;
+	    }
+	    else
+#endif
+	    {
+		    if ( ! add_auth_entry (d, &auths, NULL, NULL, FamilyInternet, (char *)&((struct sockaddr_in *)ia)->sin_addr, sizeof (struct in_addr)))
+			    goto get_local_auth_error;
+
+		    if (gdm_is_loopback_addr (&((struct sockaddr_in *)ia)->sin_addr))
+			    added_lo = TRUE;
+
+	    }
     }
 
     /* if local server add loopback */
     if (SERVER_IS_LOCAL (d) && ! added_lo && ! d->tcp_disallowed) {
-	    if ( ! add_auth_entry (d, &auths, NULL, NULL, FamilyInternet,
-				   lo, 4))
-		    goto get_local_auth_error;
+
+#ifdef ENABLE_IPV6
+	    if (d->addrtype == AF_INET6) {
+		    if ( ! add_auth_entry (d, &auths, NULL, NULL, FamilyInternet, lo6, sizeof (struct in6_addr)))
+			    goto get_local_auth_error;
+	    }
+	    else
+#endif
+	    {
+		    if ( ! add_auth_entry (d, &auths, NULL, NULL, FamilyInternet, lo, sizeof (struct in_addr)))
+			    goto get_local_auth_error;
+	    }
+
     }
 
     if G_UNLIKELY (GdmDebug)
@@ -439,9 +516,9 @@ try_open_append (const char *file)
 {
 	FILE *fp;
 
-	fp = fopen (file, "a+");
+	VE_IGNORE_EINTR (fp = fopen (file, "a+"));
 	if G_LIKELY (fp != NULL) {
-		fclose (fp);
+		VE_IGNORE_EINTR (fclose (fp));
 		return TRUE;
 	} else {
 		return FALSE;
@@ -454,18 +531,15 @@ try_open_read_as_root (const char *file)
 	int fd;
 	uid_t oldeuid = geteuid ();
 	uid_t oldegid = getegid ();
-	setegid (0);
-	seteuid (0);
+	NEVER_FAILS_root_set_euid_egid (0, 0);
 
-	fd = open (file, O_RDONLY);
+	VE_IGNORE_EINTR (fd = open (file, O_RDONLY));
 	if G_UNLIKELY (fd < 0) {
-		seteuid (oldeuid);
-		setegid (oldegid);
+		NEVER_FAILS_root_set_euid_egid (oldeuid, oldegid);
 		return FALSE;
 	} else {
-		close (fd);
-		seteuid (oldeuid);
-		setegid (oldegid);
+		VE_IGNORE_EINTR (close (fd));
+		NEVER_FAILS_root_set_euid_egid (oldeuid, oldegid);
 		return TRUE;
 	}
 }
@@ -494,6 +568,7 @@ gdm_auth_user_add (GdmDisplay *d, uid_t user, const char *homedir)
     gboolean authdir_is_tmp_dir = FALSE;
     gboolean locked;
     gboolean user_auth_exists;
+    int closeret;
 
     if (!d)
 	return FALSE;
@@ -605,7 +680,7 @@ try_user_add_again:
 
 	d->last_auth_touch = time (NULL);
 
-	af = fdopen (authfd, "w");
+	VE_IGNORE_EINTR (af = fdopen (authfd, "w"));
     }
     else { /* User's Xauthority file is ok */
 	d->authfb = FALSE;
@@ -666,7 +741,7 @@ try_user_add_again:
 			   "gdm_auth_user_add");
 
 		if ( ! d->authfb) {
-			fclose (af);
+			VE_IGNORE_EINTR (fclose (af));
 			if (locked)
 				XauUnlockAuth (d->userauth);
 			g_free (d->userauth);
@@ -682,7 +757,8 @@ try_user_add_again:
 	auths = auths->next;
     }
 
-    if G_UNLIKELY (fclose (af) < 0) {
+    VE_IGNORE_EINTR (closeret = fclose (af));
+    if G_UNLIKELY (closeret < 0) {
 	    gdm_error (_("%s: Could not write cookie"),
 		       "gdm_auth_user_add");
 
@@ -735,7 +811,7 @@ gdm_auth_user_remove (GdmDisplay *d, uid_t user)
     /* If we are using the fallback cookie location, simply nuke the
      * cookie file */
     if (d->authfb) {
-	IGNORE_EINTR (unlink (d->userauth));
+	VE_IGNORE_EINTR (unlink (d->userauth));
 	g_free (d->userauth);
 	d->userauth = NULL;
 	return;
@@ -806,9 +882,15 @@ gdm_auth_user_remove (GdmDisplay *d, uid_t user)
     af = gdm_auth_purge (d, af, TRUE /* remove when empty */);
 
     /* Close the file and unlock it */
-    if (af != NULL)
+    if (af != NULL) {
 	    /* FIXME: what about out of diskspace errors on errors close */
-	    fclose (af);
+	    errno = 0;
+	    VE_IGNORE_EINTR (fclose (af));
+	    if G_UNLIKELY (errno != 0) {
+		    gdm_error (_("Can't write to %s: %s"), d->userauth,
+			       strerror (errno));
+	    }
+    }
 
     XauUnlockAuth (d->userauth);
 
@@ -900,11 +982,11 @@ gdm_auth_purge (GdmDisplay *d, FILE *af, gboolean remove_when_empty)
 		break;
     }
 
-    fclose (af);
+    VE_IGNORE_EINTR (fclose (af));
 
     if (remove_when_empty &&
 	keep == NULL) {
-	    IGNORE_EINTR (unlink (d->userauth));
+	    VE_IGNORE_EINTR (unlink (d->userauth));
 	    return NULL;
     }
 
