@@ -322,7 +322,7 @@ gdm_display_manage (GdmDisplay *d)
 	gdm_debug ("gdm_display_manage: Forked slave: %d",
 		   (int)pid);
 	d->master_notify_fd = fds[1];
-	IGNORE_EINTR (close (fds[0]));
+	VE_IGNORE_EINTR (close (fds[0]));
 	break;
     }
 
@@ -362,6 +362,10 @@ gdm_display_unmanage (GdmDisplay *d)
     gdm_debug ("gdm_display_unmanage: Stopping %s (slave pid: %d)",
 	       d->name, (int)d->slavepid);
 
+    /* whack connections about this display */
+    if (unixconn != NULL)
+      gdm_kill_subconnections_with_display (unixconn, d);
+
     /* Kill slave, this may in fact hang for a bit at least until the
      * slave dies, which should be ASAP though */
     whack_old_slave (d);
@@ -411,6 +415,10 @@ gdm_display_dispose (GdmDisplay *d)
     if (d == NULL)
 	return;
 
+    /* paranoia */
+    if (unixconn != NULL)
+            gdm_kill_subconnections_with_display (unixconn, d);
+
     if (d->socket_conn != NULL) {
 	    GdmConnection *conn = d->socket_conn;
 	    d->socket_conn = NULL;
@@ -418,12 +426,12 @@ gdm_display_dispose (GdmDisplay *d)
     }
 
     if (d->slave_notify_fd >= 0) {
-	    IGNORE_EINTR (close (d->slave_notify_fd));
+	    VE_IGNORE_EINTR (close (d->slave_notify_fd));
 	    d->slave_notify_fd = -1;
     }
 
     if (d->master_notify_fd >= 0) {
-	    IGNORE_EINTR (close (d->master_notify_fd));
+	    VE_IGNORE_EINTR (close (d->master_notify_fd));
 	    d->master_notify_fd = -1;
     }
 
@@ -457,7 +465,7 @@ gdm_display_dispose (GdmDisplay *d)
     d->authfile_gdm = NULL;
 
     if (d->xnest_temp_auth_file != NULL) {
-	    IGNORE_EINTR (unlink (d->xnest_temp_auth_file));
+	    VE_IGNORE_EINTR (unlink (d->xnest_temp_auth_file));
     }
     g_free (d->xnest_temp_auth_file);
     d->xnest_temp_auth_file = NULL;
@@ -500,6 +508,24 @@ gdm_display_dispose (GdmDisplay *d)
     g_free (d->xsession_errors_filename);
     d->xsession_errors_filename = NULL;
 
+    if (d->session_output_fd >= 0) {
+	    VE_IGNORE_EINTR (close (d->session_output_fd));
+	    d->session_output_fd = -1;
+    }
+
+    if (d->xsession_errors_fd >= 0) {
+	    VE_IGNORE_EINTR (close (d->xsession_errors_fd));
+	    d->xsession_errors_fd = -1;
+    }
+
+    g_free (d->chooser_last_line);
+    d->chooser_last_line = NULL;
+
+    if (d->chooser_output_fd >= 0) {
+	    VE_IGNORE_EINTR (close (d->chooser_output_fd));
+	    d->chooser_output_fd = -1;
+    }
+
     g_free (d);
 }
 
