@@ -46,6 +46,7 @@ extern gchar *GdmTimedLogin;
 extern gchar *GdmUser;
 extern gboolean GdmAllowRemoteAutoLogin;
 extern gint GdmRetryDelay;
+extern gboolean GdmDisplayLastLogin;
 
 extern gboolean no_console;
 
@@ -205,6 +206,12 @@ gdm_verify_pam_conv (int num_msg, const struct pam_message **msg,
 		    /* FIXME: this is a HACK HACK HACK */
 		    g_free (tmp_PAM_USER);
 		    tmp_PAM_USER = g_strdup (s);
+
+		    if (GdmDisplayLastLogin) {
+			    char *info = gdm_get_last_info (s);
+			    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX, info);
+			    g_free (info);
+		    }
 	    }
 
 	    reply[replies].resp_retcode = PAM_SUCCESS;
@@ -504,9 +511,14 @@ authenticate_again:
 
     pam_set_item (pamh, PAM_USER_PROMPT, _("Username:"));
 
+#if 0
+    /* FIXME: this makes things wait at the wrong places! such as
+       when running the configurator.  We wish to ourselves cancel logins
+       without a delay, so ... evil */
 #ifdef PAM_FAIL_DELAY
     pam_fail_delay (pamh, GdmRetryDelay * 1000000);
 #endif /* PAM_FAIL_DELAY */
+#endif
 
     did_we_ask_for_password = FALSE;
 
@@ -544,9 +556,12 @@ authenticate_again:
 	    if (started_timer)
 		    gdm_slave_greeter_ctl_no_ret (GDM_STOPTIMER, "");
 	    if (gdm_slave_should_complain ()) {
-#ifndef PAM_FAIL_DELAY
+		    /* FIXME: see note above about PAM_FAIL_DELAY */
+/* #ifndef PAM_FAIL_DELAY */
 		    gdm_sleep_no_signal (GdmRetryDelay);
-#endif /* PAM_FAIL_DELAY */
+		    /* wait up to 100ms randomly */
+		    usleep (g_random_int_range (0, 100000));
+/* #endif */ /* PAM_FAIL_DELAY */
 		    gdm_error (_("Couldn't authenticate user"));
 	    }
 	    goto pamerr;
