@@ -62,7 +62,7 @@ static Language languages [] = {
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Basque"), "eu_ES", "Euskara", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
-	{ N_("A-M|Belarusian"), "be_BY", NULL, 0 },
+	{ N_("A-M|Belarusian"), "be_BY", "Беларуская мова", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Bengali"), "bn_BD", "বাংলা", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
@@ -76,7 +76,7 @@ static Language languages [] = {
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Chinese (simplified)"), "zh_CN", "中文 (简体)", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
-	{ N_("A-M|Chinese (traditional)"), "zh_TW", "繁體中文", 0 },
+	{ N_("A-M|Chinese (traditional)"), "zh_TW", "中文 (繁體)", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Croatian"), "hr_HR", "Hrvatski", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
@@ -142,12 +142,18 @@ static Language languages [] = {
         /*Note translate the A-M to the A-M you used in the group label */
         { N_("A-M|Malay"), "ms_MY", "Bahasa Melayu", 0 },
         /*Note translate the A-M to the A-M you used in the group label */
+        { N_("A-M|Malayalam"), "ml_IN", "മലയാളം", 0 },
+        /*Note translate the A-M to the A-M you used in the group label */
+        { N_("A-M|Marathi"), "mr_IN", NULL, 0 },
+        /*Note translate the A-M to the A-M you used in the group label */
         { N_("A-M|Mongolian"), "mn_MN", "Монгол", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Norwegian (bokmal)"), "no_NO", "Norsk (bokmål)", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Norwegian (nynorsk)"), "nn_NO", "Norsk (nynorsk)", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Oriya"), "or_IN", NULL, 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Panjabi"), "pa_IN", NULL, 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Persian"), "fa_IR", "فارسی", 0 },
@@ -425,11 +431,12 @@ gdm_lang_read_locale_file (const char *locale_file)
 	Language *language;
 	gboolean clean;
 	char *curlocale;
+	char *getsret;
 
 	if (locale_file == NULL)
 		return NULL;
 
-	langlist = fopen (locale_file, "r");
+	VE_IGNORE_EINTR (langlist = fopen (locale_file, "r"));
 
 	if (langlist == NULL)
 		return NULL;
@@ -438,12 +445,16 @@ gdm_lang_read_locale_file (const char *locale_file)
 
 	dupcheck = g_hash_table_new (g_str_hash, g_str_equal);
 
-	while (fgets (curline, sizeof (curline), langlist)) {
+	for (;;) {
 		char *name;
 		char *lang;
 		char **lang_list;
 		int i;
 
+		VE_IGNORE_EINTR (getsret = fgets (curline, sizeof (curline), langlist));
+		if (getsret == NULL)
+			break;
+
 		if (curline[0] <= ' ' ||
 		    curline[0] == '#')
 			continue;
@@ -516,7 +527,7 @@ gdm_lang_read_locale_file (const char *locale_file)
 
 	langs = g_list_sort (langs, lang_collate);
 
-	fclose (langlist);
+	VE_IGNORE_EINTR (fclose (langlist));
 
 	return langs;
 }
