@@ -5,6 +5,7 @@
 #include <string.h>
 
 #include "gdmwm.h"
+#include "gdmcommon.h"
 #include "gdmlanguages.h"
 #include "greeter.h"
 #include "greeter_configuration.h"
@@ -49,7 +50,7 @@ greeter_langauge_initialize_model (void)
 
   gtk_list_store_append (lang_model, &iter);
   gtk_list_store_set (lang_model, &iter,
-		      TRANSLATED_NAME_COLUMN, _("System default"),
+		      TRANSLATED_NAME_COLUMN, _("System Default"),
 		      UNTRANSLATED_NAME_COLUMN, NULL,
 		      LOCALE_COLUMN, DEFAULT_LANGUAGE,
 		      -1);
@@ -90,7 +91,7 @@ greeter_language_init (void)
 {
   static gboolean initted = FALSE;
 
-  g_assert (initted == FALSE);
+  g_assert ( ! initted);
     
   greeter_langauge_initialize_model ();
 
@@ -129,7 +130,7 @@ greeter_language_get_language (const char *old_language)
 	  char *current_name, *saved_name;
 
 	  if (strcmp (current_language, DEFAULT_LANGUAGE) == 0)
-	    current_name = g_strdup (_("System default"));
+	    current_name = g_strdup (_("System Default"));
 	  else
 	    current_name = gdm_lang_name (current_language,
 					  FALSE /* never_encoding */,
@@ -137,7 +138,7 @@ greeter_language_get_language (const char *old_language)
 					  TRUE /* untranslated */,
 					  TRUE /* markup */);
 	  if (strcmp (old_language, "") == 0)
-	    saved_name = g_strdup (_("System default"));
+	    saved_name = g_strdup (_("System Default"));
 	  else
 	    saved_name = gdm_lang_name (old_language,
 					FALSE /* never_encoding */,
@@ -151,7 +152,7 @@ greeter_language_get_language (const char *old_language)
 	  g_free (current_name);
 	  g_free (saved_name);
 
-	  savelang = greeter_query (msg, _("Make _Default"), _("Just For _This Session"));
+	  savelang = gdm_common_query (msg, TRUE /* markup */, _("Make _Default"), _("Just For _This Session"));
 	  g_free (msg);
 	}
     }
@@ -174,6 +175,23 @@ selection_changed (GtkTreeSelection *selection,
   gtk_tree_model_get (GTK_TREE_MODEL (lang_model), &iter, LOCALE_COLUMN, &dialog_selected_language, -1);
 }
 
+static void
+tree_row_activated (GtkTreeView         *view,
+                    GtkTreePath         *path,
+                    GtkTreeViewColumn   *column,
+                    gpointer            data)
+{
+  GtkTreeIter iter;
+  if (gtk_tree_model_get_iter (GTK_TREE_MODEL (lang_model), &iter, path))
+    {
+      g_free (dialog_selected_language);
+      gtk_tree_model_get (GTK_TREE_MODEL (lang_model), &iter,
+			  LOCALE_COLUMN, &dialog_selected_language,
+			  -1);
+      gtk_dialog_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+  }
+}
+
 void
 greeter_action_language (GreeterItemInfo *info,
 			 gpointer         user_data)
@@ -241,6 +259,10 @@ greeter_action_language (GreeterItemInfo *info,
 			"changed",
 			(GCallback) selection_changed,
 			NULL);
+      g_signal_connect(G_OBJECT (view),
+                        "row_activated",
+                        (GCallback) tree_row_activated,
+                        NULL);
       gtk_widget_show_all (dialog);
       gdm_wm_center_window (GTK_WINDOW (dialog));
     }
